@@ -28,20 +28,21 @@ import abc
 import contextlib
 import functools
 import os
+import platform
 import selectors
 import signal
 import socket
 import sys
 import typing
 
-from urwid import signals, util
+from urwid import signals, str_util, util
 
 from . import escape
 from .common import UNPRINTABLE_TRANS_TABLE, UPDATE_PALETTE_ENTRY, AttrSpec, BaseScreen, RealTerminal
 
 if typing.TYPE_CHECKING:
     import io
-    from collections.abc import Callable
+    from collections.abc import Callable, Iterable
     from types import FrameType
 
     from typing_extensions import Literal
@@ -49,10 +50,11 @@ if typing.TYPE_CHECKING:
     from urwid import Canvas, EventLoop
 
 IS_WINDOWS = sys.platform == "win32"
+IS_WSL = (sys.platform == "linux") and ("wsl" in platform.platform().lower())
 
 
 class Screen(BaseScreen, RealTerminal):
-    def __init__(self, input: io.IOBase, output: io.IOBase):  # noqa: A002
+    def __init__(self, input: io.IOBase, output: io.IOBase):  # noqa: A002  # pylint: disable=redefined-builtin
         """Initialize a screen that directly prints escape codes to an output
         terminal.
         """
@@ -171,7 +173,7 @@ class Screen(BaseScreen, RealTerminal):
         After calling this function get_input will include mouse
         click events along with keystrokes.
         """
-        enable = bool(enable)
+        enable = bool(enable)  # noqa: FURB123,RUF100
         if enable == self._mouse_tracking_enabled:
             return
 
@@ -227,12 +229,10 @@ class Screen(BaseScreen, RealTerminal):
         self._term_output_file.flush()
 
     @typing.overload
-    def get_input(self, raw_keys: Literal[False]) -> list[str]:
-        ...
+    def get_input(self, raw_keys: Literal[False]) -> list[str]: ...
 
     @typing.overload
-    def get_input(self, raw_keys: Literal[True]) -> tuple[list[str], list[int]]:
-        ...
+    def get_input(self, raw_keys: Literal[True]) -> tuple[list[str], list[int]]: ...
 
     def get_input(self, raw_keys: bool = False) -> list[str] | tuple[list[str], list[int]]:
         """Return pending input as a list.
@@ -355,8 +355,7 @@ class Screen(BaseScreen, RealTerminal):
 
     def _make_legacy_input_wrapper(self, event_loop, callback):
         """
-        Support old Screen classes that still have a get_input_nonblocking and
-        expect it to work.
+        Support old Screen classes that still have a get_input_nonblocking and expect it to work.
         """
 
         @functools.wraps(callback)
@@ -364,7 +363,7 @@ class Screen(BaseScreen, RealTerminal):
             if self._input_timeout:
                 event_loop.remove_alarm(self._input_timeout)
                 self._input_timeout = None
-            timeout, keys, raw = self.get_input_nonblocking()
+            timeout, keys, raw = self.get_input_nonblocking()  # pylint: disable=no-member  # should we deprecate?
             if timeout is not None:
                 self._input_timeout = event_loop.alarm(timeout, wrapper)
 
@@ -373,7 +372,7 @@ class Screen(BaseScreen, RealTerminal):
         return wrapper
 
     def _get_input_codes(self) -> list[int]:
-        return self._get_keyboard_codes()
+        return list(self._get_keyboard_codes())
 
     def get_available_raw_input(self) -> list[int]:
         """
@@ -406,8 +405,7 @@ class Screen(BaseScreen, RealTerminal):
         callback: None,
         codes: list[int],
         wait_for_more: bool = ...,
-    ) -> tuple[list[str], list[int]]:
-        ...
+    ) -> tuple[list[str], list[int]]: ...
 
     @typing.overload
     def parse_input(
@@ -416,8 +414,7 @@ class Screen(BaseScreen, RealTerminal):
         callback: None,
         codes: list[int],
         wait_for_more: bool = ...,
-    ) -> tuple[list[str], list[int]]:
-        ...
+    ) -> tuple[list[str], list[int]]: ...
 
     @typing.overload
     def parse_input(
@@ -426,8 +423,7 @@ class Screen(BaseScreen, RealTerminal):
         callback: Callable[[list[str], list[int]], typing.Any],
         codes: list[int],
         wait_for_more: bool = ...,
-    ) -> None:
-        ...
+    ) -> None: ...
 
     def parse_input(
         self,
@@ -496,15 +492,6 @@ class Screen(BaseScreen, RealTerminal):
         # For get_input
         return decoded_codes, raw_codes
 
-    def _get_keyboard_codes(self) -> list[int]:
-        codes = []
-        while True:
-            code = self._getch_nodelay()
-            if code < 0:
-                break
-            codes.append(code)
-        return codes
-
     def _wait_for_input_ready(self, timeout: float | None) -> list[int]:
         logger = self.logger.getChild("wait_for_input_ready")
         fd_list = self.get_input_descriptors()
@@ -521,11 +508,10 @@ class Screen(BaseScreen, RealTerminal):
         return [event.fd for event, _ in ready]
 
     @abc.abstractmethod
-    def _getch(self, timeout: int) -> int:
-        ...
+    def _read_raw_input(self, timeout: int) -> Iterable[int]: ...
 
-    def _getch_nodelay(self) -> int:
-        return self._getch(0)
+    def _get_keyboard_codes(self) -> Iterable[int]:
+        return self._read_raw_input(0)
 
     def _setup_G1(self) -> None:
         """
@@ -541,49 +527,9 @@ class Screen(BaseScreen, RealTerminal):
                 break
         self._setup_G1_done = True
 
-    def draw_screen(self, size: tuple[int, int], r: Canvas) -> None:
+    def draw_screen(self, size: tuple[int, int], canvas: Canvas) -> None:
         """Paint screen with rendered canvas."""
 
-        logger = self.logger.getChild("draw_screen")
-
-        (maxcol, maxrow) = size
-
-        if not self._started:
-            raise RuntimeError
-
-        if maxrow != r.rows():
-            raise ValueError(maxrow)
-
-        # quick return if nothing has changed
-        if self.screen_buf and r is self._screen_buf_canvas:
-            return
-
-        self._setup_G1()
-
-        if self._resized:
-            # handle resize before trying to draw screen
-            logger.debug("Not drawing screen: screen resized and resize was not handled")
-            return
-
-        logger.debug(f"Drawing screen with size {size!r}")
-
-        o = [escape.HIDE_CURSOR, self._attrspec_to_escape(AttrSpec("", ""))]
-
-        def partial_display() -> bool:
-            # returns True if the screen is in partial display mode ie. only some rows belong to the display
-            return self._rows_used is not None
-
-        if not partial_display():
-            o.append(escape.CURSOR_HOME)
-
-        if self.screen_buf:
-            osb = self.screen_buf
-        else:
-            osb = []
-        sb = []
-        cy = self._cy
-        y = -1
-
         def set_cursor_home() -> str:
             if not partial_display():
                 return escape.set_cursor_position(0, 0)
@@ -596,7 +542,7 @@ class Screen(BaseScreen, RealTerminal):
                 return "\b" + escape.CURSOR_HOME_COL + escape.move_cursor_up(cy - y) + escape.move_cursor_right(x)
             return "\b" + escape.CURSOR_HOME_COL + escape.move_cursor_down(y - cy) + escape.move_cursor_right(x)
 
-        def is_blank_row(row) -> bool:
+        def is_blank_row(row: list[tuple[object, Literal["0", "U"] | None], bytes]) -> bool:
             if len(row) > 1:
                 return False
             if row[0][2].strip():
@@ -609,17 +555,65 @@ class Screen(BaseScreen, RealTerminal):
             if isinstance(a, AttrSpec):
                 return self._attrspec_to_escape(a)
             # undefined attributes use default/default
-            # TODO: track and report these
+            self.logger.debug(f"Undefined attribute: {a!r}")
             return self._attrspec_to_escape(AttrSpec("default", "default"))
 
-        def using_standout_or_underline(a):
+        def using_standout_or_underline(a: AttrSpec | str) -> bool:
             a = self._pal_attrspec.get(a, a)
             return isinstance(a, AttrSpec) and (a.standout or a.underline)
 
+        encoding = util.get_encoding()
+
+        logger = self.logger.getChild("draw_screen")
+
+        (maxcol, maxrow) = size
+
+        if not self._started:
+            raise RuntimeError
+
+        if maxrow != canvas.rows():
+            raise ValueError(maxrow)
+
+        # quick return if nothing has changed
+        if self.screen_buf and canvas is self._screen_buf_canvas:
+            return
+
+        self._setup_G1()
+
+        if self._resized:
+            # handle resize before trying to draw screen
+            logger.debug("Not drawing screen: screen resized and resize was not handled")
+            return
+
+        logger.debug(f"Drawing screen with size {size!r}")
+
+        last_attributes = None  # Default = empty
+
+        output: list[str] = [escape.HIDE_CURSOR, attr_to_escape(last_attributes)]
+
+        def partial_display() -> bool:
+            # returns True if the screen is in partial display mode ie. only some rows belong to the display
+            return self._rows_used is not None
+
+        if not partial_display():
+            output.append(escape.CURSOR_HOME)
+
+        if self.screen_buf:
+            osb = self.screen_buf
+        else:
+            osb = []
+        sb: list[list[tuple[object, Literal["0", "U"] | None, bytes]]] = []
+        cy = self._cy
+        y = -1
+
         ins = None
-        o.append(set_cursor_home())
+        output.append(set_cursor_home())
         cy = 0
-        for row in r.content():
+
+        first = True
+        last_charset_flag = None
+
+        for row in canvas.content():
             y += 1
             if osb and y < len(osb) and osb[y] == row:
                 # this row of the screen buffer matches what is
@@ -637,7 +631,7 @@ class Screen(BaseScreen, RealTerminal):
                 self._rows_used = y
 
             if y or partial_display():
-                o.append(set_cursor_position(0, y))
+                output.append(set_cursor_position(0, y))
             # after updating the line we will be just over the
             # edge, but terminals still treat this as being
             # on the same line
@@ -652,61 +646,78 @@ class Screen(BaseScreen, RealTerminal):
                 elif y == maxrow - 1 and maxcol > 1:
                     row, back, ins = self._last_row(row)  # noqa: PLW2901
 
-            first = True
-            lasta = lastcs = None
             for a, cs, run in row:
-                if not isinstance(run, bytes):  # canvases should render with bytes
+                if not isinstance(run, bytes):  # canvases render with bytes
                     raise TypeError(run)
+
                 if cs != "U":
                     run = run.translate(UNPRINTABLE_TRANS_TABLE)  # noqa: PLW2901
-                if first or lasta != a:
-                    o.append(attr_to_escape(a))
-                    lasta = a
-                if first or lastcs != cs:
+
+                if last_attributes != a:
+                    output.append(attr_to_escape(a))
+                    last_attributes = a
+
+                if encoding != "utf-8" and (first or last_charset_flag != cs):
                     if cs not in {None, "0", "U"}:
                         raise ValueError(cs)
-                    if lastcs == "U":
-                        o.append(escape.IBMPC_OFF)
+                    if last_charset_flag == "U":
+                        output.append(escape.IBMPC_OFF)
 
                     if cs is None:
-                        o.append(escape.SI)
+                        output.append(escape.SI)
                     elif cs == "U":
-                        o.append(escape.IBMPC_ON)
+                        output.append(escape.IBMPC_ON)
                     else:
-                        o.append(escape.SO)
-                    lastcs = cs
-                o.append(run)
+                        output.append(escape.SO)
+                    last_charset_flag = cs
+
+                output.append(run.decode(encoding, "replace"))
                 first = False
+
             if ins:
                 (inserta, insertcs, inserttext) = ins
                 ias = attr_to_escape(inserta)
                 if insertcs not in {None, "0", "U"}:
                     raise ValueError(insertcs)
-                if cs is None:
-                    icss = escape.SI
-                elif cs == "U":
-                    icss = escape.IBMPC_ON
+
+                if isinstance(inserttext, bytes):
+                    inserttext = inserttext.decode(encoding)
+
+                output.extend(("\x08" * back, ias))
+
+                if encoding != "utf-8":
+                    if cs is None:
+                        icss = escape.SI
+                    elif cs == "U":
+                        icss = escape.IBMPC_ON
+                    else:
+                        icss = escape.SO
+
+                    output.append(icss)
+
+                if not IS_WINDOWS:
+                    output += [escape.INSERT_ON, inserttext, escape.INSERT_OFF]
                 else:
-                    icss = escape.SO
-                o += ["\x08" * back, ias, icss, escape.INSERT_ON, inserttext, escape.INSERT_OFF]
+                    output += [f"{escape.ESC}[{str_util.calc_width(inserttext, 0, len(inserttext))}@", inserttext]
+
+                if encoding != "utf-8" and cs == "U":
+                    output.append(escape.IBMPC_OFF)
 
-                if cs == "U":
-                    o.append(escape.IBMPC_OFF)
             if whitespace_at_end:
-                o.append(escape.ERASE_IN_LINE_RIGHT)
+                output.append(escape.ERASE_IN_LINE_RIGHT)
 
-        if r.cursor is not None:
-            x, y = r.cursor
-            o += [set_cursor_position(x, y), escape.SHOW_CURSOR]
+        if canvas.cursor is not None:
+            x, y = canvas.cursor
+            output += [set_cursor_position(x, y), escape.SHOW_CURSOR]
             self._cy = y
 
         if self._resized:
             # handle resize before trying to draw screen
             return
         try:
-            for line in o:
+            for line in output:
                 if isinstance(line, bytes):
-                    line = line.decode("utf-8", "replace")  # noqa: PLW2901
+                    line = line.decode(encoding, "replace")  # noqa: PLW2901
                 self.write(line)
             self.flush()
         except OSError as e:
@@ -715,9 +726,13 @@ class Screen(BaseScreen, RealTerminal):
                 raise
 
         self.screen_buf = sb
-        self._screen_buf_canvas = r
+        self._screen_buf_canvas = canvas
 
-    def _last_row(self, row):
+    def _last_row(self, row: list[tuple[object, Literal["0", "U"] | None, bytes]]) -> tuple[
+        list[tuple[object, Literal["0", "U"] | None, bytes]],
+        int,
+        tuple[object, Literal["0", "U"] | None, bytes],
+    ]:
         """On the last row we need to slide the bottom right character
         into place. Calculate the new line, attr and an insert sequence
         to do that.
@@ -730,24 +745,24 @@ class Screen(BaseScreen, RealTerminal):
 
         new_row = row[:-1]
         z_attr, z_cs, last_text = row[-1]
-        last_cols = util.calc_width(last_text, 0, len(last_text))
-        last_offs, z_col = util.calc_text_pos(last_text, 0, len(last_text), last_cols - 1)
+        last_cols = str_util.calc_width(last_text, 0, len(last_text))
+        last_offs, z_col = str_util.calc_text_pos(last_text, 0, len(last_text), last_cols - 1)
         if last_offs == 0:
             z_text = last_text
             del new_row[-1]
             # we need another segment
             y_attr, y_cs, nlast_text = row[-2]
-            nlast_cols = util.calc_width(nlast_text, 0, len(nlast_text))
+            nlast_cols = str_util.calc_width(nlast_text, 0, len(nlast_text))
             z_col += nlast_cols
-            nlast_offs, y_col = util.calc_text_pos(nlast_text, 0, len(nlast_text), nlast_cols - 1)
+            nlast_offs, y_col = str_util.calc_text_pos(nlast_text, 0, len(nlast_text), nlast_cols - 1)
             y_text = nlast_text[nlast_offs:]
             if nlast_offs:
                 new_row.append((y_attr, y_cs, nlast_text[:nlast_offs]))
         else:
             z_text = last_text[last_offs:]
             y_attr, y_cs = z_attr, z_cs
-            nlast_cols = util.calc_width(last_text, 0, last_offs)
-            nlast_offs, y_col = util.calc_text_pos(last_text, 0, last_offs, nlast_cols - 1)
+            nlast_cols = str_util.calc_width(last_text, 0, last_offs)
+            nlast_offs, y_col = str_util.calc_text_pos(last_text, 0, last_offs, nlast_cols - 1)
             y_text = last_text[nlast_offs:last_offs]
             if nlast_offs:
                 new_row.append((y_attr, y_cs, last_text[:nlast_offs]))
