@@ -118,7 +118,7 @@ class MainLoop:
         screen: BaseScreen | None = None,
         handle_mouse: bool = True,
         input_filter: Callable[[list[str], list[int]], list[str]] | None = None,
-        unhandled_input: Callable[[str | tuple[str, int, int, int]], bool] | None = None,
+        unhandled_input: Callable[[str | tuple[str, int, int, int]], bool | None] | None = None,
         event_loop: EventLoop | None = None,
         pop_ups: bool = False,
     ):
@@ -209,6 +209,8 @@ class MainLoop:
         :param callback: function to call with two parameters: this main loop
                          object and *user_data*
         :type callback: callable
+        :param user_data: optional user data to pass to the callback
+        :type user_data: object
         """
         self.logger.debug(f"Setting alarm in {sec!r} seconds with callback {callback!r}")
 
@@ -228,6 +230,8 @@ class MainLoop:
         :param callback: function to call with two parameters: this main loop
                          object and *user_data*
         :type callback: callable
+        :param user_data: optional user data to pass to the callback
+        :type user_data: object
         """
         sec = tm - time.time()
         self.logger.debug(f"Setting alarm in {sec!r} seconds with callback {callback!r}")
@@ -282,8 +286,7 @@ class MainLoop:
 
             def cb() -> None:
                 data = os.read(pipe_rd, PIPE_BUFFER_READ_SIZE)
-                rval = callback(data)
-                if rval is False:
+                if not callback(data):
                     self.event_loop.remove_watch_file(watch_handle)
                     os.close(pipe_rd)
 
@@ -471,12 +474,11 @@ class MainLoop:
 
     def _run_screen_event_loop(self) -> None:
         """
-        This method is used when the screen does not support using
-        external event loops.
+        This method is used when the screen does not support using external event loops.
 
-        The alarms stored in the SelectEventLoop in :attr:`event_loop`
-        are modified by this method.
+        The alarms stored in the SelectEventLoop in :attr:`event_loop` are modified by this method.
         """
+        # pylint: disable=protected-access  # special case for alarms handling
         self.logger.debug(f"Starting screen {self.screen!r} event loop")
 
         next_alarm = None
@@ -622,7 +624,7 @@ class MainLoop:
             return self._input_filter(keys, raw)
         return keys
 
-    def unhandled_input(self, data: str | tuple[str, int, int, int]) -> bool:
+    def unhandled_input(self, data: str | tuple[str, int, int, int]) -> bool | None:
         """
         This function is called with any input that was not handled by the
         widgets, and calls the *unhandled_input* function passed to the
