@@ -76,7 +76,8 @@ class ScrollSupportingBody(Sized, Protocol):
     def get_prev(self, position: _K) -> tuple[Widget, _K] | tuple[None, None]: ...
 
 
-class ListWalker(metaclass=signals.MetaSignals):
+class ListWalker(metaclass=signals.MetaSignals):  # pylint: disable=no-member, unsubscriptable-object
+    # mixin not named as mixin
     signals: typing.ClassVar[list[str]] = ["modified"]
 
     def _modified(self) -> None:
@@ -481,9 +482,7 @@ class ListBox(Widget, WidgetContainerMixin):
                 break
             fill_lines -= p_rows
 
-        trim_bottom = focus_rows + offset_rows - inset_rows - maxrow
-        if trim_bottom < 0:
-            trim_bottom = 0
+        trim_bottom = max(focus_rows + offset_rows - inset_rows - maxrow, 0)
 
         # 3. collect the widgets below the focus
         pos = focus_pos
@@ -548,9 +547,7 @@ class ListBox(Widget, WidgetContainerMixin):
         if size is not None:
             self._rendered_size = size
 
-        _, focus_pos = self._body.get_focus()
-
-        mid, top, bottom = self.calculate_visible(self._rendered_size, focus)
+        mid, top, _bottom = self.calculate_visible(self._rendered_size, focus)
 
         start_row = top.trim
         maxcol = self._rendered_size[0]
@@ -779,6 +776,8 @@ class ListBox(Widget, WidgetContainerMixin):
 
     def _contents(self):
         class ListBoxContents:
+            # pylint: disable=no-self-argument
+
             __getitem__ = self._contents__getitem__
 
             __len__ = self.__len__
@@ -1204,15 +1203,13 @@ class ListBox(Widget, WidgetContainerMixin):
 
         return key
 
-    def _keypress_max_left(self, size: tuple[int, int]) -> bool:
+    def _keypress_max_left(self, size: tuple[int, int]) -> None:
         self.focus_position = next(iter(self.body.positions()))
         self.set_focus_valign(VAlign.TOP)
-        return True
 
-    def _keypress_max_right(self, size: tuple[int, int]) -> bool:
+    def _keypress_max_right(self, size: tuple[int, int]) -> None:
         self.focus_position = next(iter(self.body.positions(reverse=True)))
         self.set_focus_valign(VAlign.BOTTOM)
-        return True
 
     def _keypress_up(self, size: tuple[int, int]) -> bool | None:
         (maxcol, maxrow) = size
