@@ -39,7 +39,7 @@ class MetaSignals(type):
         signals = d.get("signals", [])
         for superclass in cls.__bases__:
             signals.extend(getattr(superclass, "signals", []))
-        signals = list({x: None for x in signals}.keys())
+        signals = list(dict.fromkeys(signals).keys())
         d["signals"] = signals
         register_signal(cls, signals)
         super().__init__(name, bases, d)
@@ -70,8 +70,8 @@ class Signals:
 
     def register(self, sig_cls, signals: Container[Hashable]) -> None:
         """
-        :param sig_class: the class of an object that will be sending signals
-        :type sig_class: class
+        :param sig_cls: the class of an object that will be sending signals
+        :type sig_cls: class
         :param signals: a list of signals that may be sent, typically each
                         signal is represented by a string
         :type signals: signal names
@@ -180,8 +180,7 @@ class Signals:
         # Just generate an arbitrary (but unique) key
         key = Key()
 
-        signals = setdefaultattr(obj, self._signal_attr, {})
-        handlers = signals.setdefault(name, [])
+        handlers = setdefaultattr(obj, self._signal_attr, {}).setdefault(name, [])
 
         # Remove the signal handler when any of the weakref'd arguments
         # are garbage collected. Note that this means that the handlers
@@ -195,7 +194,7 @@ class Signals:
         # their callbacks) from existing.
         obj_weak = weakref.ref(obj)
 
-        def weakref_callback(weakref):
+        def weakref_callback(weakref):  # pylint: disable=redefined-outer-name  # bad, but not changing API
             o = obj_weak()
             if o:
                 self.disconnect_by_key(o, name, key)
@@ -277,8 +276,7 @@ class Signals:
         If the callback is not connected or already disconnected, this
         function will simply do nothing.
         """
-        signals = setdefaultattr(obj, self._signal_attr, {})
-        handlers = signals.get(name, [])
+        handlers = setdefaultattr(obj, self._signal_attr, {}).get(name, [])
         handlers[:] = [h for h in handlers if h[0] is not key]
 
     def emit(self, obj, name: Hashable, *args) -> bool:
@@ -295,8 +293,7 @@ class Signals:
         This function returns True if any of the callbacks returned True.
         """
         result = False
-        signals = getattr(obj, self._signal_attr, {})
-        handlers = signals.get(name, [])
+        handlers = getattr(obj, self._signal_attr, {}).get(name, [])
         for _key, callback, user_arg, (weak_args, user_args) in handlers:
             result |= self._call_callback(callback, user_arg, weak_args, user_args, args)
         return result
