@@ -1,16 +1,42 @@
 from __future__ import annotations
 
+import enum
+import typing
+
 from urwid.canvas import CompositeCanvas, SolidCanvas
 
-from .constants import Sizing
+from .constants import BOX_SYMBOLS, SHADE_SYMBOLS, Sizing
 from .widget import Widget
 
 
+class DividerSymbols(str, enum.Enum):
+    """Common symbols for divider widgets."""
+
+    # Lines
+    LIGHT_HL = BOX_SYMBOLS.LIGHT.HORIZONTAL
+    LIGHT_4_DASHES = BOX_SYMBOLS.LIGHT.HORIZONTAL_4_DASHES
+    LIGHT_3_DASHES = BOX_SYMBOLS.LIGHT.HORIZONTAL_3_DASHES
+    LIGHT_2_DASHES = BOX_SYMBOLS.LIGHT.HORIZONTAL_2_DASHES
+    HEAVY_HL = BOX_SYMBOLS.HEAVY.HORIZONTAL
+    HEAVY_4_DASHES = BOX_SYMBOLS.HEAVY.HORIZONTAL_4_DASHES
+    HEAVY_3_DASHES = BOX_SYMBOLS.HEAVY.HORIZONTAL_3_DASHES
+    HEAVY_2_DASHES = BOX_SYMBOLS.HEAVY.HORIZONTAL_2_DASHES
+    DOUBLE_HL = BOX_SYMBOLS.DOUBLE.HORIZONTAL
+
+    # Full block
+    FULL_BLOCK = SHADE_SYMBOLS.FULL_BLOCK
+    DARK_SHADE = SHADE_SYMBOLS.DARK_SHADE
+    MEDIUM_SHADE = SHADE_SYMBOLS.MEDIUM_SHADE
+    LITE_SHADE = SHADE_SYMBOLS.LITE_SHADE
+
+
 class Divider(Widget):
     """
     Horizontal divider widget
     """
 
+    Symbols = DividerSymbols
+
     _sizing = frozenset([Sizing.FLOW])
 
     ignore_focus = True
@@ -43,10 +69,10 @@ class Divider(Widget):
         self.top = top
         self.bottom = bottom
 
-    def _repr_words(self):
+    def _repr_words(self) -> list[str]:
         return super()._repr_words() + [repr(self.div_char)] * (self.div_char != " ")
 
-    def _repr_attrs(self):
+    def _repr_attrs(self) -> dict[str, typing.Any]:
         attrs = dict(super()._repr_attrs())
         if self.top:
             attrs["top"] = self.top
@@ -66,7 +92,7 @@ class Divider(Widget):
         (_maxcol,) = size
         return self.top + 1 + self.bottom
 
-    def render(self, size: tuple[int], focus: bool = False):
+    def render(self, size: tuple[int], focus: bool = False) -> CompositeCanvas:
         """
         Render the divider as a canvas and return it.
 
