@@ -7,16 +7,19 @@ from urwid import text_layout
 from urwid.canvas import CompositeCanvas
 from urwid.command_map import Command
 from urwid.split_repr import remove_defaults
-from urwid.util import decompose_tagmarkup, is_wide_char, move_next_char, move_prev_char
-
-if typing.TYPE_CHECKING:
-    from typing_extensions import Literal
-
-    from urwid.canvas import TextCanvas
+from urwid.str_util import is_wide_char, move_next_char, move_prev_char
+from urwid.util import decompose_tagmarkup
 
 from .constants import Align, Sizing, WrapMode
 from .text import Text, TextError
 
+if typing.TYPE_CHECKING:
+    from collections.abc import Hashable
+
+    from typing_extensions import Literal
+
+    from urwid.canvas import TextCanvas
+
 
 class EditError(TextError):
     pass
@@ -62,15 +65,15 @@ class Edit(Text):
 
     def __init__(
         self,
-        caption="",
-        edit_text: str | bytes = "",
+        caption: str | tuple[Hashable, str] | list[str | tuple[Hashable, str]] = "",
+        edit_text: str = "",
         multiline: bool = False,
         align: Literal["left", "center", "right"] | Align = Align.LEFT,
         wrap: Literal["space", "any", "clip", "ellipsis"] | WrapMode = WrapMode.SPACE,
         allow_tab: bool = False,
         edit_pos: int | None = None,
-        layout=None,
-        mask: str | bytes | None = None,
+        layout: text_layout.TextLayout = None,
+        mask: str | None = None,
     ) -> None:
         """
         :param caption: markup for caption preceding edit_text, see
@@ -110,7 +113,7 @@ class Edit(Text):
         self._edit_pos = 0
         self._caption, self._attrib = decompose_tagmarkup(caption)
         self._edit_text = ""
-        self.highlight = None
+        self.highlight: tuple[int, int] | None = None
         self.set_edit_text(edit_text)
         if edit_pos is None:
             edit_pos = len(edit_text)
@@ -127,10 +130,10 @@ class Edit(Text):
         )
 
     def _repr_attrs(self) -> dict[str, typing.Any]:
-        attrs = dict(super()._repr_attrs(), edit_pos=self._edit_pos)
+        attrs = {**super()._repr_attrs(), "edit_pos": self._edit_pos}
         return remove_defaults(attrs, Edit.__init__)
 
-    def get_text(self):
+    def get_text(self) -> tuple[str | bytes, list[tuple[Hashable, int]]]:
         """
         Returns ``(text, display attributes)``. See :meth:`Text.get_text`
         for details.
@@ -150,7 +153,7 @@ class Edit(Text):
 
         return self._caption + (self._mask * len(self._edit_text)), self._attrib
 
-    def set_text(self, markup) -> None:
+    def set_text(self, markup: tuple[str, list[tuple[Hashable, int]]]) -> None:
         """
         Not supported by Edit widget.
 
@@ -206,7 +209,7 @@ class Edit(Text):
 
         return pref_col
 
-    def set_caption(self, caption) -> None:
+    def set_caption(self, caption: str | tuple[Hashable, str] | list[str | tuple[Hashable, str]]) -> None:
         """
         Set the caption markup for this widget.
 
@@ -257,10 +260,7 @@ class Edit(Text):
         >>> e.edit_pos
         4
         """
-        if pos < 0:
-            pos = 0
-        if pos > len(self._edit_text):
-            pos = len(self._edit_text)
+        pos = min(max(pos, 0), len(self._edit_text))
         self.highlight = None
         self.pref_col_maxcol = None, None
         self._edit_pos = pos
@@ -274,7 +274,7 @@ class Edit(Text):
         """,
     )
 
-    def set_mask(self, mask: str | bytes | None) -> None:
+    def set_mask(self, mask: str | None) -> None:
         """
         Set the character for masking text away.
 
@@ -285,7 +285,7 @@ class Edit(Text):
         self._mask = mask
         self._invalidate()
 
-    def set_edit_text(self, text: str | bytes) -> None:
+    def set_edit_text(self, text: str) -> None:
         """
         Set the edit text for this widget.
 
@@ -333,7 +333,7 @@ class Edit(Text):
         """,
     )
 
-    def insert_text(self, text: str | bytes) -> None:
+    def insert_text(self, text: str) -> None:
         """
         Insert text at the cursor position and update cursor.
         This method is used by the keypress() method when inserting
@@ -371,7 +371,7 @@ class Edit(Text):
             return text.encode("ascii")  # follow python2's implicit conversion
         return text.decode("ascii")
 
-    def insert_text_result(self, text: str | bytes) -> tuple[str | bytes, int]:
+    def insert_text_result(self, text: str) -> tuple[str | bytes, int]:
         """
         Return result of insert_text(text) without actually performing the
         insertion.  Handy for pre-validation.
@@ -384,7 +384,7 @@ class Edit(Text):
         # if there's highlighted text, it'll get replaced by the new text
         text = self._normalize_to_caption(text)
         if self.highlight:
-            start, stop = self.highlight
+            start, stop = self.highlight  # pylint: disable=unpacking-non-sequence  # already checked
             btext, etext = self.edit_text[:start], self.edit_text[stop:]
             result_text = btext + etext
             result_pos = start
@@ -418,8 +418,6 @@ class Edit(Text):
         >>> e.keypress(size, 'shift f1')
         'shift f1'
         """
-        (maxcol,) = size
-
         pos = self.edit_pos
         if self.valid_char(key):
             if isinstance(key, str) and not isinstance(self._caption, str):
@@ -457,8 +455,8 @@ class Edit(Text):
         if self._command_map[key] in {Command.UP, Command.DOWN}:
             self.highlight = None
 
-            _x, y = self.get_cursor_coords((maxcol,))
-            pref_col = self.get_pref_col((maxcol,))
+            _x, y = self.get_cursor_coords(size)
+            pref_col = self.get_pref_col(size)
             if pref_col is None:
                 raise ValueError(pref_col)
 
@@ -470,7 +468,7 @@ class Edit(Text):
             else:
                 y += 1
 
-            if not self.move_cursor_to_coords((maxcol,), pref_col, y):
+            if not self.move_cursor_to_coords(size, pref_col, y):
                 return key
             return None
 
@@ -499,12 +497,12 @@ class Edit(Text):
             self.highlight = None
             self.pref_col_maxcol = None, None
 
-            _x, y = self.get_cursor_coords((maxcol,))
+            _x, y = self.get_cursor_coords(size)
 
             if self._command_map[key] == Command.MAX_LEFT:
-                self.move_cursor_to_coords((maxcol,), Align.LEFT, y)
+                self.move_cursor_to_coords(size, Align.LEFT, y)
             else:
-                self.move_cursor_to_coords((maxcol,), Align.RIGHT, y)
+                self.move_cursor_to_coords(size, Align.RIGHT, y)
             return None
 
         # key wasn't handled
@@ -540,17 +538,21 @@ class Edit(Text):
             return False
 
         pos = text_layout.calc_pos(self.get_text()[0], trans, x, y)
-        e_pos = pos - len(self.caption)
-        if e_pos < 0:
-            e_pos = 0
-        if e_pos > len(self.edit_text):
-            e_pos = len(self.edit_text)
+        e_pos = min(max(pos - len(self.caption), 0), len(self.edit_text))
         self.edit_pos = e_pos
         self.pref_col_maxcol = x, maxcol
         self._invalidate()
         return True
 
-    def mouse_event(self, size: tuple[int], event: str, button: int, x: int, y: int, focus: bool) -> bool | None:
+    def mouse_event(
+        self,
+        size: tuple[int],
+        event: str,
+        button: int,
+        col: int,
+        row: int,
+        focus: bool,
+    ) -> bool | None:
         """
         Move the cursor to the location clicked for button 1.
 
@@ -561,9 +563,8 @@ class Edit(Text):
         >>> e.edit_pos
         2
         """
-        (maxcol,) = size
         if button == 1:
-            return self.move_cursor_to_coords((maxcol,), x, y)
+            return self.move_cursor_to_coords(size, col, row)
         return False
 
     def _delete_highlighted(self) -> bool:
@@ -573,7 +574,7 @@ class Edit(Text):
         """
         if not self.highlight:
             return False
-        start, stop = self.highlight
+        start, stop = self.highlight  # pylint: disable=unpacking-non-sequence  # already checked
         btext, etext = self.edit_text[:start], self.edit_text[stop:]
         self.set_edit_text(btext + etext)
         self.edit_pos = start
@@ -592,9 +593,9 @@ class Edit(Text):
         >>> c.cursor
         (5, 0)
         """
-        self._shift_view_to_cursor = bool(focus)
+        self._shift_view_to_cursor = bool(focus)  # noqa: FURB123,RUF100
 
-        canv: TextCanvas | CompositeCanvas = super().render(size)
+        canv: TextCanvas | CompositeCanvas = super().render(size, focus)
         if focus:
             canv = CompositeCanvas(canv)
             canv.cursor = self.get_cursor_coords(size)
@@ -605,7 +606,11 @@ class Edit(Text):
         #    d.coords['highlight'] = [ hstart, hstop ]
         return canv
 
-    def get_line_translation(self, maxcol: int, ta=None):
+    def get_line_translation(
+        self,
+        maxcol: int,
+        ta: tuple[str | bytes, list[tuple[Hashable, int]]] | None = None,
+    ) -> list[list[tuple[int, int, int | bytes] | tuple[int, int | None]]]:
         trans = super().get_line_translation(maxcol, ta)
         if not self._shift_view_to_cursor:
             return trans
@@ -640,7 +645,7 @@ class Edit(Text):
         self._shift_view_to_cursor = True
         return self.position_coords(maxcol, self.edit_pos)
 
-    def position_coords(self, maxcol: int, pos) -> tuple[int, int]:
+    def position_coords(self, maxcol: int, pos: int) -> tuple[int, int]:
         """
         Return (*x*, *y*) coordinates for an offset into self.edit_text.
         """
