@@ -29,12 +29,12 @@ from urwid.text_layout import calc_coords
 from urwid.util import is_mouse_press
 
 from .columns import Columns
-from .constants import Align, Sizing, WrapMode
+from .constants import Align, WrapMode
 from .text import Text
-from .widget import WidgetWrap
+from .widget import WidgetError, WidgetWrap
 
 if typing.TYPE_CHECKING:
-    from collections.abc import Callable, MutableSequence
+    from collections.abc import Callable, Hashable, MutableSequence
 
     from typing_extensions import Literal, Self
 
@@ -50,7 +50,7 @@ class SelectableIcon(Text):
 
     def __init__(
         self,
-        text,
+        text: str | tuple[Hashable, str] | list[str | tuple[Hashable, str]],
         cursor_position: int = 0,
         align: Literal["left", "center", "right"] | Align = Align.LEFT,
         wrap: Literal["space", "any", "clip", "ellipsis"] | WrapMode = WrapMode.SPACE,
@@ -75,7 +75,11 @@ class SelectableIcon(Text):
         super().__init__(text, align=align, wrap=wrap, layout=layout)
         self._cursor_position = cursor_position
 
-    def render(self, size: tuple[int] | tuple[()], focus: bool = False) -> TextCanvas | CompositeCanvas:  # type: ignore[override]
+    def render(
+        self,
+        size: tuple[int] | tuple[()],
+        focus: bool = False,
+    ) -> TextCanvas | CompositeCanvas:  # type: ignore[override]
         """
         Render the text content of this widget with a cursor when
         in focus.
@@ -130,14 +134,11 @@ class SelectableIcon(Text):
         return key
 
 
-class CheckBoxError(Exception):
+class CheckBoxError(WidgetError):
     pass
 
 
-class CheckBox(WidgetWrap):
-    def sizing(self):
-        return frozenset([Sizing.FLOW])
-
+class CheckBox(WidgetWrap[Columns]):
     states: typing.ClassVar[dict[bool | Literal["mixed"], SelectableIcon]] = {
         True: SelectableIcon("[X]", 1),
         False: SelectableIcon("[ ]", 1),
@@ -153,54 +154,50 @@ class CheckBox(WidgetWrap):
     @typing.overload
     def __init__(
         self,
-        label,
+        label: str | tuple[Hashable, str] | list[str | tuple[Hashable, str]],
         state: bool = False,
         has_mixed: typing.Literal[False] = False,
         on_state_change: Callable[[Self, bool, _T], typing.Any] | None = None,
         user_data: _T = ...,
         checked_symbol: str | None = ...,
-    ) -> None:
-        ...
+    ) -> None: ...
 
     @typing.overload
     def __init__(
         self,
-        label,
+        label: str | tuple[Hashable, str] | list[str | tuple[Hashable, str]],
         state: bool = False,
         has_mixed: typing.Literal[False] = False,
         on_state_change: Callable[[Self, bool], typing.Any] | None = None,
         user_data: None = None,
         checked_symbol: str | None = ...,
-    ) -> None:
-        ...
+    ) -> None: ...
 
     @typing.overload
     def __init__(
         self,
-        label: str,
+        label: str | tuple[Hashable, str] | list[str | tuple[Hashable, str]],
         state: typing.Literal["mixed"] | bool = False,
         has_mixed: typing.Literal[True] = True,
         on_state_change: Callable[[Self, bool | typing.Literal["mixed"], _T], typing.Any] | None = None,
         user_data: _T = ...,
         checked_symbol: str | None = ...,
-    ) -> None:
-        ...
+    ) -> None: ...
 
     @typing.overload
     def __init__(
         self,
-        label: str,
+        label: str | tuple[Hashable, str] | list[str | tuple[Hashable, str]],
         state: typing.Literal["mixed"] | bool = False,
         has_mixed: typing.Literal[True] = True,
         on_state_change: Callable[[Self, bool | typing.Literal["mixed"]], typing.Any] | None = None,
         user_data: None = None,
         checked_symbol: str | None = ...,
-    ) -> None:
-        ...
+    ) -> None: ...
 
     def __init__(
         self,
-        label,
+        label: str | tuple[Hashable, str] | list[str | tuple[Hashable, str]],
         state: bool | Literal["mixed"] = False,
         has_mixed: typing.Literal[False, True] = False,  # MyPy issue: Literal[True, False] is not equal `bool`
         on_state_change: (
@@ -235,12 +232,12 @@ class CheckBox(WidgetWrap):
           urwid.disconnect_signal(check_box, 'change', callback, user_data)
 
         >>> CheckBox("Confirm")
-        <CheckBox selectable flow widget 'Confirm' state=False>
+        <CheckBox selectable fixed/flow widget 'Confirm' state=False>
         >>> CheckBox("Yogourt", "mixed", True)
-        <CheckBox selectable flow widget 'Yogourt' state='mixed'>
+        <CheckBox selectable fixed/flow widget 'Yogourt' state='mixed'>
         >>> cb = CheckBox("Extra onions", True)
         >>> cb
-        <CheckBox selectable flow widget 'Extra onions' state=True>
+        <CheckBox selectable fixed/flow widget 'Extra onions' state=True>
         >>> cb.render((20,), focus=True).text
         [b'[X] Extra onions    ']
         >>> CheckBox("Test", None)
@@ -270,7 +267,7 @@ class CheckBox(WidgetWrap):
             ),
         )
 
-    def pack(self, size: tuple[int] | None = None, focus: bool = False) -> tuple[str, str]:
+    def pack(self, size: tuple[()] | tuple[int] | None = None, focus: bool = False) -> tuple[str, str]:
         """Pack for widget.
 
         :param size: size data. Special case: None - get minimal widget size to fit
@@ -284,21 +281,18 @@ class CheckBox(WidgetWrap):
         >>> ml_cb = CheckBox("Multi\\nline\\ncheckbox")
         >>> ml_cb.pack()
         (12, 3)
+        >>> ml_cb.pack((), True)
+        (12, 3)
         """
-        if size is not None:
-            return super().pack(size, focus)
-
-        width, _height = self._label.pack()
-        width += self.reserve_columns
-        return width, self.rows((width,))
+        return super().pack(size or (), focus)
 
     def _repr_words(self) -> list[str]:
         return [*super()._repr_words(), repr(self.label)]
 
-    def _repr_attrs(self):
-        return dict(super()._repr_attrs(), state=self.state)
+    def _repr_attrs(self) -> dict[str, typing.Any]:
+        return {**super()._repr_attrs(), "state": self.state}
 
-    def set_label(self, label):
+    def set_label(self, label: str | tuple[Hashable, str] | list[str | tuple[Hashable, str]]):
         """
         Change the check box label.
 
@@ -307,10 +301,10 @@ class CheckBox(WidgetWrap):
 
         >>> cb = CheckBox(u"foo")
         >>> cb
-        <CheckBox selectable flow widget 'foo' state=False>
+        <CheckBox selectable fixed/flow widget 'foo' state=False>
         >>> cb.set_label(('bright_attr', u"bar"))
         >>> cb
-        <CheckBox selectable flow widget 'bar' state=False>
+        <CheckBox selectable fixed/flow widget 'bar' state=False>
         """
         self._label.set_text(label)
         # no need to call self._invalidate(). WidgetWrap takes care of
@@ -440,7 +434,7 @@ class CheckBox(WidgetWrap):
         elif self.state == "mixed":
             self.set_state(False)
 
-    def mouse_event(self, size: tuple[int], event, button: int, x: int, y: int, focus: bool) -> bool:
+    def mouse_event(self, size: tuple[int], event: str, button: int, x: int, y: int, focus: bool) -> bool:
         """
         Toggle state on button 1 press.
 
@@ -470,29 +464,27 @@ class RadioButton(CheckBox):
     @typing.overload
     def __init__(
         self,
-        group: MutableSequence[CheckBox],
-        label,
+        group: MutableSequence[RadioButton],
+        label: str | tuple[Hashable, str] | list[str | tuple[Hashable, str]],
         state: bool | Literal["first True"] = ...,
         on_state_change: Callable[[Self, bool, _T], typing.Any] | None = None,
         user_data: _T = ...,
-    ) -> None:
-        ...
+    ) -> None: ...
 
     @typing.overload
     def __init__(
         self,
-        group: MutableSequence[CheckBox],
-        label,
+        group: MutableSequence[RadioButton],
+        label: str | tuple[Hashable, str] | list[str | tuple[Hashable, str]],
         state: bool | Literal["first True"] = ...,
         on_state_change: Callable[[Self, bool], typing.Any] | None = None,
         user_data: None = None,
-    ) -> None:
-        ...
+    ) -> None: ...
 
     def __init__(
         self,
-        group: MutableSequence[CheckBox],
-        label,
+        group: MutableSequence[RadioButton],
+        label: str | tuple[Hashable, str] | list[str | tuple[Hashable, str]],
         state: bool | Literal["first True"] = "first True",
         on_state_change: Callable[[Self, bool, _T], typing.Any] | Callable[[Self, bool], typing.Any] | None = None,
         user_data: _T | None = None,
@@ -527,9 +519,9 @@ class RadioButton(CheckBox):
         >>> len(bgroup)
         2
         >>> b1
-        <RadioButton selectable flow widget 'Agree' state=True>
+        <RadioButton selectable fixed/flow widget 'Agree' state=True>
         >>> b2
-        <RadioButton selectable flow widget 'Disagree' state=False>
+        <RadioButton selectable fixed/flow widget 'Disagree' state=False>
         >>> b2.render((15,), focus=True).text # ... = b in Python 3
         [...'( ) Disagree   ']
         """
@@ -564,10 +556,10 @@ class RadioButton(CheckBox):
         ...     radio_button.set_label(u"Think Harder!")
         >>> key = connect_signal(b3, 'change', relabel_button)
         >>> b3
-        <RadioButton selectable flow widget 'Unsure' state=False>
+        <RadioButton selectable fixed/flow widget 'Unsure' state=False>
         >>> b3.set_state(True) # this will trigger the callback
         >>> b3
-        <RadioButton selectable flow widget 'Think Harder!' state=True>
+        <RadioButton selectable fixed/flow widget 'Think Harder!' state=True>
         """
         if self._state == state:
             return
@@ -583,8 +575,8 @@ class RadioButton(CheckBox):
         for cb in self.group:
             if cb is self:
                 continue
-            if cb._state:
-                cb.set_state(False)
+            if cb.state:
+                cb.state = False
 
     def toggle_state(self) -> None:
         """
@@ -605,10 +597,7 @@ class RadioButton(CheckBox):
         self.set_state(True)
 
 
-class Button(WidgetWrap):
-    def sizing(self):
-        return frozenset([Sizing.FLOW])
-
+class Button(WidgetWrap[Columns]):
     button_left = Text("<")
     button_right = Text(">")
 
@@ -617,32 +606,30 @@ class Button(WidgetWrap):
     @typing.overload
     def __init__(
         self,
-        label,
+        label: str | tuple[Hashable, str] | list[str | tuple[Hashable, str]],
         on_press: Callable[[Self, _T], typing.Any] | None = None,
         user_data: _T = ...,
         *,
         align: Literal["left", "center", "right"] | Align = ...,
         wrap: Literal["space", "any", "clip", "ellipsis"] | WrapMode = ...,
         layout: TextLayout | None = ...,
-    ) -> None:
-        ...
+    ) -> None: ...
 
     @typing.overload
     def __init__(
         self,
-        label,
+        label: str | tuple[Hashable, str] | list[str | tuple[Hashable, str]],
         on_press: Callable[[Self], typing.Any] | None = None,
         user_data: None = None,
         *,
         align: Literal["left", "center", "right"] | Align = ...,
         wrap: Literal["space", "any", "clip", "ellipsis"] | WrapMode = ...,
         layout: TextLayout | None = ...,
-    ) -> None:
-        ...
+    ) -> None: ...
 
     def __init__(
         self,
-        label,
+        label: str | tuple[Hashable, str] | list[str | tuple[Hashable, str]],
         on_press: Callable[[Self, _T], typing.Any] | Callable[[Self], typing.Any] | None = None,
         user_data: _T | None = None,
         *,
@@ -677,7 +664,7 @@ class Button(WidgetWrap):
 
         >>> from urwid.util import set_temporary_encoding
         >>> Button(u"Ok")
-        <Button selectable flow widget 'Ok'>
+        <Button selectable fixed/flow widget 'Ok'>
         >>> b = Button("Cancel")
         >>> b.render((15,), focus=True).text # ... = b in Python 3
         [b'< Cancel      >']
@@ -701,7 +688,7 @@ class Button(WidgetWrap):
         if on_press:
             connect_signal(self, "click", on_press, user_data)
 
-    def pack(self, size: tuple[int] | None = None, focus: bool = False) -> tuple[int, int]:
+    def pack(self, size: tuple[()] | tuple[int] | None = None, focus: bool = False) -> tuple[int, int]:
         """Pack for widget.
 
         :param size: size data. Special case: None - get minimal widget size to fit
@@ -712,19 +699,16 @@ class Button(WidgetWrap):
         (10, 2)
         >>> btn.pack()
         (15, 1)
+        >>> btn.pack((), True)
+        (15, 1)
         """
-        if size is not None:
-            return super().pack(size, focus)
-
-        width, _height = self._label.pack()
-        width += 4  # We use Given sizing for button ends and `dividechars=1`
-        return width, self.rows((width,))
+        return super().pack(size or (), focus)
 
     def _repr_words(self) -> list[str]:
         # include button.label in repr(button)
         return [*super()._repr_words(), repr(self.label)]
 
-    def set_label(self, label) -> None:
+    def set_label(self, label: str | tuple[Hashable, str] | list[str | tuple[Hashable, str]]) -> None:
         """
         Change the button label.
 
@@ -733,7 +717,7 @@ class Button(WidgetWrap):
         >>> b = Button("Ok")
         >>> b.set_label(u"Yup yup")
         >>> b
-        <Button selectable flow widget 'Yup yup'>
+        <Button selectable fixed/flow widget 'Yup yup'>
         """
         self._label.set_text(label)
 
@@ -774,7 +758,7 @@ class Button(WidgetWrap):
         self._emit("click")
         return None
 
-    def mouse_event(self, size: tuple[int], event, button: int, x: int, y: int, focus: bool) -> bool:
+    def mouse_event(self, size: tuple[int], event: str, button: int, x: int, y: int, focus: bool) -> bool:
         """
         Send 'click' signal on button 1 press.
 
